####################################################################
# Monte Carlo Test
# Vanilla SE vs. Cluster SE vs. Cluster-Adjusted SE
# 
# Multinomial logit Model
####################################################################

clusterIMtestMNL<-function(obs.per.cluster, g.sd, reps=1000){
  
  require(mlogit)
  require(foreign)
  total.obs <- sum(obs.per.cluster)
  n.clusters <- length(obs.per.cluster)

  vn.res <- matrix(data=NA, nrow=reps, ncol=6)
  cl.res <- matrix(data=NA, nrow=reps, ncol=6)
  mx.res <- matrix(data=NA, nrow=reps, ncol=6)
  im.res <- matrix(data=NA, nrow=reps, ncol=6)
  bs.res.v <- matrix(data=NA, nrow=reps, ncol=6)
  bs.res.c <- matrix(data=NA, nrow=reps, ncol=6)
  
  cl.mlogit   <- function(fm, cluster){
    
    # fm: a fitted mlogit model
    # cluster: a data vector with the cluster
    #          identity of each observation in fm
    
    #require(sandwich, quietly = TRUE)
    #require(lmtest, quietly = TRUE)
    M <- length(unique(cluster))
    N <- length(cluster)
    K <- length(coefficients(fm))
    dfc <- (M/(M-1))
    uj  <- apply(estfun(fm),2, function(x) tapply(x, cluster, sum));
    vcovCL <- dfc*sandwich(fm, meat.=crossprod(uj)/N)
    coeftest(fm, vcovCL, df=(M-1)) 
  }
  
  pb <- txtProgressBar(min=0, max=reps, initial=0, style=3)
  for(i in 1:reps){

    setTxtProgressBar(pb, value=i)
    
    mod <- NA
    
    # while loop protects against problem data sets
    while(max(is.na(mod))){   
    
      g <- rep(1:n.clusters, times=obs.per.cluster)
      g.ef.1 <- rnorm(n.clusters, mean = 0, sd = g.sd)
      g.ob.1 <- g.ef.1[g]
      g.ef.2 <- rnorm(n.clusters, mean = 0, sd = g.sd)
      g.ob.2 <- g.ef.2[g]
      
      x.mean <- runif(n.clusters, min = 1, max = 5)
      x <- rnorm( total.obs, mean = x.mean[g], sd = 1 )
      z <- rnorm( total.obs, mean = 0, sd = 1)
      w <- rnorm( total.obs, mean = 0, sd = 1)
      
      den <- exp(0)+exp(g.ob.1 + w)+exp(g.ob.2-1.5*w)
      y.2.prob <- exp(g.ob.1 + w) / den
      y.3.prob <- exp(g.ob.2 - 1.5*w) / den
      mnp.probs <- cbind((1-y.2.prob-y.3.prob), y.2.prob, y.3.prob)
      y<-c()
      for(j in 1:total.obs){
        temp <- rmultinom(1, size=1, prob=mnp.probs[j,])
        y[j] <- which(temp==1)
      }
      
      dat.t<-data.frame(y,x,z,w,g)
#     write.dta(dat.t, file="simdat_temp.dta")
      dat <- mlogit.data(dat.t, choice = "y", shape="wide")
      
      mod <- tryCatch((mlogit(y ~ 0 | x+z+w, data=dat)),  error = function(e){NA})
      
      if(length(unique(y)) < 3){mod <- NA}  # reject any data sets that have no variation in the DV
      
    }
 
    vn.res[i,] <- summary(mod)$CoefTable[3:8,4]
    cl.res[i,] <- cl.mlogit(mod, g)[3:8,4]
    mx.res[i,] <- apply(rbind(vn.res[i,], cl.res[i,]), MARGIN=2, FUN=max)
    im.res[i,] <- tryCatch(cluster.im.mlogit(mod, dat, ~ g, report=F, truncate=T)$p.values[3:8,],  error = function(e){NA})
    bs.res.v[i,] <- cluster.bs.mlogit(mod, dat, ~ g, report = F, boot.reps=100, prog.bar=F, cluster.se = FALSE)$p.values[3:8,]
    bs.res.c[i,] <- cluster.bs.mlogit(mod, dat, ~ g, report = F, boot.reps=100, prog.bar=F, cluster.se = TRUE)$p.values[3:8,]

  }
  close(pb)

  save.name <- paste("multinomial-trunc-clusters-", as.character(n.clusters), "_unique-", as.character(length(unique(obs.per.cluster))), "_sd-", g.sd, ".RData", sep="")
  list.out <- list(vn.res=vn.res, cl.res=cl.res, mx.res=mx.res, im.res=im.res, bs.res.v=bs.res.v, bs.res.c=bs.res.c)
  save(list.out, file=save.name)
  

}
